/******************************************************************/
/*                                                                */
/* Program: plotcocoon2.c, version 1.05                           */
/*                                                                */
/* First version:                October 2016                     */
/* Latest update:                  2018-02-06                     */
/*                                                                */
/* Program by:        Stefan Spaennare, Lund, Sweden              */
/* E-mail:            stefan.sp@outlook.com                       */
/* Home page:         http:/www.spaennare.se                      */
/* Project web page:  http://www.spaennare.se/tmplotter           */
/*                                                                */
/* Description:                                                   */
/*                                                                */
/* This program plots or mills a cocoon curve.                    */
/*                                                                */
/* The program suggests suitable input parameters. First run      */
/* the program without using a pen or milling tool.               */
/*                                                                */
/* Read the readme.txt document as a whole before you start to    */
/* use the program. The program uses the file stepcfg2.txt that   */
/* contains stepper motor aspect ratio information for the curve. */
/* This must be calibrated before starting to use the program.    */
/*                                                                */
/* It is assumed that you have a Velleman K8055 USB I/O card to   */
/* your computer connected to your stepper motor drives. This     */
/* I/O card use +5 V logic. If your stepper motor drives have     */
/* opto coupler inputs you need a +5 V driver circuit in between. */
/*                                                                */
/* The stepper motor drives are connected to the digital output   */
/* of the K8055 I/O card as follows. Each digital output needs    */
/* a pull up resistor (330 Ohm) connected to +5 V.                */
/*                                                                */
/* GND: Signal ground.                                            */
/*                                                                */
/* DI 01: Stepper motor 1 clock/step.                             */
/* DI 02: Stepper motor 1 dir cw/ccw.                             */
/* DI 03: Stepper motor 1 enable (active high).                   */
/*                                                                */
/* DI 04: Stepper motor 2 clock/step.                             */
/* DI 05: Stepper motor 2 dir cw/ccw.                             */
/* DI 06: Stepper motor 2 enable (active high).                   */
/*                                                                */
/******************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/



#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <time.h>
#include <windows.h>
#include ".\K8055D.h"


double timetic=(double)(CLOCKS_PER_SEC);

char cfg_name[21]="stepcfg2.txt";


double harm_delay(long delay0) {

   long j;
   double sum;


      sum=0.0;

      for (j=1; j<=delay0; j++) {
         sum=sum+1.0/(double)(j);
      } /* for j */

      return(sum);

} /* harm_delay */


int main() {

	FILE *cfgfile;

	int Row, Col;
	int BoardNum = 0;
	int ULStat = 0;
	long DataValue;
	int BitLevel;
	long BitNum;
	long BitValue;
	int hilo;
	char s[32];  // string for user input

	char cc;

	long i, j,k,d0,d1,ri,stepsx,stepsy,dirx,diry,cardadr;
	long delay0;

	double sum, sum0,v,v0,dv,r0,vr,r,v1,mmx,mmy,ccrad1;
	double x1, y1,x2,y2,dx,dy,x0,y0,ex,ey,rr,r1;
	double x7, y7, aex, aey,dr,xfac,yfac;

	double pi1 = 3.14159265358979323;



	DataValue = 8;

	cardadr=0;
	OpenDevice(cardadr);


	for (j = 1; j <= 8; j++) {

	   BitNum = j;
	   BitValue = 0;

	   SetDigitalChannel(BitNum);

    } /* for j */


	BitNum = 3;
	BitValue = 1;

	ClearDigitalChannel(BitNum);


	BitNum = 6;
	BitValue = 1;

	ClearDigitalChannel(BitNum);


	if ((cfgfile = fopen(cfg_name, "r")) == NULL) {
		printf("File %s not found.\n", cfg_name);
		exit(0);
	} /* if */

	if ((cfgfile = fopen(cfg_name, "r")) == NULL) {
		printf("File %s not found.\n", cfg_name);
		exit(0);
	} /* if */

	fscanf(cfgfile,"%ld %lf %ld", &stepsx, &mmx, &dirx);
	fscanf(cfgfile,"%ld %lf %ld", &stepsy, &mmy, &diry);

	if ((dirx != 0) && (dirx != 1)) {
		printf("\n\ndirx not 0 or 1. Program terminated.\n\n");
		exit(0);
	} /* if */

	if ((diry != 0) && (diry != 1)) {
		printf("\n\ndiry not 0 or 1. Program terminated.\n\n");
		exit(0);
	} /* if */
	
	printf("\nCocoon Curve\n");
	printf("------------\n\n");
	printf("\nInput Cocoon Curve radius (float mm): ");
	scanf("%lf", &ccrad1);
	printf("\nInput stepper motor delay (int 0 ms): ");
	scanf("%ld", &delay0);
	printf("\n\n");

        printf("Cocoon curve radius x:       %9.3lf mm\n", ccrad1);
	printf("Stepper motor delay: %6ld ms\n\n", delay0);
	printf("stepsx mmx dirx:     %6ld %9.3lf %3ld\n", stepsx, mmx, dirx);
	printf("stepsy mmy diry:     %6ld %9.3lf %3ld\n", stepsy, mmy, diry);

	cc = 1;

	printf("\nStart milling or plotting (Y/n return)? ");

	scanf("%s", &cc);

	if (cc != 'Y') {
		printf("\n\nProgram terminated.\n\n");
		exit(0);
	} /* if */
    
	d0 = 1000;
	sum0 = 0.0;

	r1 = 0.0;
	v1 = 0.0;
	vr = 76.0*pi1;

	xfac = (ccrad1/65.0)*0.0005*(float)(stepsx)/mmx;
	yfac = (ccrad1/65.0)*0.0005*(float)(stepsy)/mmy;

	x0 = xfac*0.0;
	y0 = yfac*0.0;
	x1 = x0;
	y1 = y0;

	k = 0;


    while (v1<vr) {

		if ((k % 100) == 0) {
			printf("%8ld %12.5lf %12.3lf %12.3lf %12.3lf\n", k, v1, x1, y1, r1);
		} /* if */

		x2 = xfac*r1*cos(v1) + x0;
		y2 = yfac*r1*sin(v1*1.03) + y0;

		dx = x2 - x1;
		dy = y2 - y1;

		rr = sqrt(dx*dx + dy*dy);

		if (rr > 0.0) {
			ex = dx / rr;
			ey = dy / rr;
		}
		else {
			ex = 0.0;
			ey = 0.0;
		} /* if */

		aex = fabs(ex);
		aey = fabs(ey);

		if (dx >= 0.0) {
                        BitNum = 2;
			BitValue = dirx;

			if (BitValue == 1) {
				ClearDigitalChannel(BitNum);
			} 
			else {
				SetDigitalChannel(BitNum);
			} /* if */
		}
		else {
                 	BitNum = 2;
			BitValue = 1-dirx;

			if (BitValue == 1) {
				ClearDigitalChannel(BitNum);
			} 
			else {
				SetDigitalChannel(BitNum);
			} /* if */

		} /* if */

		if (dy >= 0.0) {
                        BitNum = 5;
			BitValue = diry;

			if (BitValue == 1) {
				ClearDigitalChannel(BitNum);
			} 
			else {
				SetDigitalChannel(BitNum);
			} /* if  */
		}
		else {
			
			BitNum = 5;
			BitValue = 1-diry;

			if (BitValue == 1) {
				ClearDigitalChannel(BitNum);
			} 
			else {
				SetDigitalChannel(BitNum);
			} /* if */ 

		} /* if */


		ri = (long)(rr);
		x7 = 0.0;
		y7 = 0.0;

		for (i = 1; i < ri; i++) {

			x7 = x7 + aex;
			y7 = y7 + aey;

			if (x7 > 1.0) {

				x7 = x7-1.0;

                                BitNum = 1;
				BitValue = 1;

				ClearDigitalChannel(BitNum);

				/* sum0 = sum0 + harm_delay(delay1 * d0); */

				Sleep(delay0);

				BitNum = 1;
				BitValue = 0;

				SetDigitalChannel(BitNum);

				/* sum0 = sum0 + harm_delay(delay1 * d0); */

				Sleep(delay0);

			} /* if */

			if (y7 > 1.0) {

				y7 = y7-1.0;

                                BitNum = 4;
				BitValue = 1;

				ClearDigitalChannel(BitNum);

				/* sum0 = sum0 + harm_delay(delay2 * d0); */

				Sleep(delay0);

				BitNum = 4;
				BitValue = 0;

				SetDigitalChannel(BitNum);

				/* sum0 = sum0 + harm_delay(delay2 * d0); */

				Sleep(delay0);


			} /* if */


		} /* for i */

		x1 = x1 + (double)(ri)*ex;
		y1 = y1 + (double)(ri)*ey;

		v1 = v1 + 0.01*pi1*0.5;
		r1 = r1 + 22.5*0.025*50.0*1.55*14.0/40.0*0.5;

		k++;


    } /* while */


	printf("%8ld %12.5lf %12.3lf %12.3lf %12.3lf\n\n", k, v1, x1, y1, r1);


        BitNum = 3;
	BitValue = 0;

	SetDigitalChannel(BitNum);

	BitNum = 6;
	BitValue = 0;

	SetDigitalChannel(BitNum);


	for (j = 1; j <= 8; j++) {

		BitNum = j;
		BitValue = 0;

	        SetDigitalChannel(BitNum);

	} /* for j */

        printf("\n%12.5lf\n",sum0);

	fclose(cfgfile);


    } /* End program */



